@@ -32,14 +32,15 @@ def set_environ():
 	tmpdir = './tests/tmp/'
 	os.environ.update({
 		'TMP': tmpdir,
-		'XDG_DATA_HOME': './tests/tmp/share',
-		'XDG_DATA_DIRS': './tests/tmp/share',
-		'XDG_CONFIG_HOME': './tests/tmp/config',
-		'XDG_CONFIG_DIRS': './tests/tmp/config',
-		'XDG_CACHE_HOME': './tests/tmp/cache'
+		'XDG_DATA_HOME': './tests/tmp/data_home',
+		'XDG_DATA_DIRS': './tests/tmp/data_dir',
+		'XDG_CONFIG_HOME': './tests/tmp/config_home',
+		'XDG_CONFIG_DIRS': './tests/tmp/config_dir',
+		'XDG_CACHE_HOME': './tests/tmp/cache_home'
 	})
-	if not os.path.isdir(tmpdir):
-		os.mkdir(tmpdir)
+	if os.path.isdir(tmpdir):
+		shutil.rmtree(unicode(tmpdir))
+	os.mkdir(tmpdir)
 
 
 def create_tmp_dir(name):
@@ -90,7 +91,7 @@ def _get_test_data_wiki():
 	for node in tree.getiterator(tag='page'):
 		name = node.attrib['name']
 		text = unicode(node.text.lstrip('\n'))
-		test_data.append((name, text)) 
+		test_data.append((name, text))
 	return tuple(test_data)
 
 
