@@ -18,6 +18,7 @@ if sys.version_info >= (2, 6):
 else:
 	import simplejson as json # extra dependency
 
+import zim.fs
 from zim.fs import *
 from zim.errors import Error
 from zim.parsing import TextBuffer, split_quoted_strings
@@ -45,10 +46,10 @@ def _set_basedirs():
 	global XDG_CACHE_HOME
 
 	# Detect if we are running from the source dir
-	if os.path.isfile('./zim.py'):
+	if zim.fs.isfile('./zim.py'):
 		scriptdir = '.' # maybe running module in test / debug
 	else:
-		scriptdir = os.path.dirname(sys.argv[0])
+		scriptdir = os.path.dirname(os.path.abspath(sys.argv[0]))
 	zim_data_dir = Dir(scriptdir + '/data')
 	if zim_data_dir.exists():
 		ZIM_DATA_DIR = zim_data_dir
@@ -176,8 +177,6 @@ def config_file(path, klass=None):
 		return klass(file, default=default)
 	elif path[-1].endswith('.conf'):
 		return ConfigDictFile(file, default=default)
-	#~ elif path[-1].endswith('.list'):
-		#~ return ConfigListFile(file, default=default)
 	else:
 		return TextConfigFile(file, default=default)
 
@@ -240,6 +239,13 @@ class ListDict(dict):
 				if isinstance(v, ListDict):
 					v.set_modified(False)
 
+	def update(D, E=None, **F):
+		if E and hasattr(E, 'keys'):
+			for k in E: D[k] = E[k]
+		elif E:
+			for (k, v) in E: D[k] = v
+		for k in F: D[k] = F[k]
+
 	def __setitem__(self, k, v):
 		dict.__setitem__(self, k, v)
 		self._modified = True
@@ -270,14 +276,25 @@ class ListDict(dict):
 		'''
 		if not k in self:
 			self.__setitem__(k, v)
-		elif check is None:
-			klass = klass or v.__class__
+		elif check:
+			if not check(self[k]):
+				logger.warn(
+					'Invalid config value for %s: "%s"', k, self[k])
+				self.__setitem__(k, v)
+		else:
+			if klass is None:
+				if v is None:
+					return self[k] # noting we can do
+				else:
+					klass = v.__class__
+
 			if issubclass(klass, basestring):
 				klass = basestring
-			if not self[k] is None and not v is None \
+
+			if not (self[k] is None and v is None) \
 			and not isinstance(self[k], klass):
 				if klass is tuple and isinstance(self[k], list):
-					# Needed because json does not know difference list or tuple
+					# Special case because json does not know difference list or tuple
 					v = tuple(self[k])
 					self.__setitem__(k, v)
 				else:
@@ -285,13 +302,15 @@ class ListDict(dict):
 						'Invalid config value for %s: "%s" - should be of type %s',
 						k, self[k], klass)
 					self.__setitem__(k, v)
-		else:
-			if not check(self[k]):
-				logger.warn(
-					'Invalid config value for %s: "%s"', k, self[k])
+			elif klass is basestring and v and not self[k]:
+				# Special case to avoid empty string
 				self.__setitem__(k, v)
+
 		return self[k]
 
+	def keys(self):
+		return self.order[:]
+
 	def items(self):
 		return map(lambda k: (k, self[k]), self.order)
 
@@ -342,50 +361,6 @@ class ListDict(dict):
 				and isinstance(value[1], int)  )
 
 
-#~ class ConfigList(ListDict):
-	#~ '''This class supports config files that exist of two columns separated
-	#~ by whitespace. It inherits from ListDict to ensure the list remain in
-	#~ the same order when it is written to file again. When a file path is set
-	#~ for this object it will be used to try reading from any from the config
-	#~ and data directories while using the config home directory for writing.
-	#~ '''
-#~
-	#~ _fields_re = re.compile(r'(?:\\.|\S)+') # match escaped char or non-whitespace
-	#~ _escaped_re = re.compile(r'\\(.)') # match single escaped char
-	#~ _escape_re = re.compile(r'([\s\\])') # match chars to escape
-#~
-	#~ def parse(self, text):
-		#~ if isinstance(text, basestring):
-			#~ text = text.splitlines(True)
-#~
-		#~ for line in text:
-			#~ line = line.strip()
-			#~ if line.isspace() or line.startswith('#'):
-				#~ continue
-			#~ cols = self._fields_re.findall(line)
-			#~ if len(cols) == 1:
-				#~ cols.append(None) # empty string in second column
-				#~ cols[0] = self._escaped_re.sub(r'\1', cols[0])
-			#~ else:
-				#~ assert len(cols) >= 2
-				#~ if len(cols) > 2 and not cols[2].startswith('#'):
-					#~ logger.warn('trailing data') # FIXME better warning
-				#~ cols[0] = self._escaped_re.sub(r'\1', cols[0])
-				#~ cols[1] = self._escaped_re.sub(r'\1', cols[1])
-			#~ self[cols[0]] = cols[1]
-#~
-	#~ def dump(self):
-		#~ text = TextBuffer()
-		#~ for k, v in self.items():
-			#~ k = self._escape_re.sub(r'\\\1', k)
-			#~ if v is None:
-				#~ v = ''
-			#~ else:
-				#~ v = self._escape_re.sub(r'\\\1', v)
-			#~ text.append("%s\t%s\n" % (k, v))
-		#~ return text.get_lines()
-
-
 class ConfigDict(ListDict):
 	'''Config object which wraps a dict of dicts.
 	These are represented as INI files where each sub-dict is a section.
@@ -411,7 +386,7 @@ class ConfigDict(ListDict):
 				section = self[name]
 			elif '=' in line:
 				parameter, value = line.split('=', 1)
-				parameter = parameter.rstrip()
+				parameter = str(parameter.rstrip()) # no unicode
 				try:
 					value = self._decode_value(value.lstrip())
 					section[parameter] = value
@@ -481,8 +456,10 @@ class ConfigFile(ListDict):
 	def read(self):
 		# TODO: flush dict first ?
 		if self.file.exists():
+			logger.debug('Loading %s', self.file.path)
 			self.parse(self.file.readlines())
 		elif self.default:
+			logger.debug('Loading %s', self.default.path)
 			self.parse(self.default.readlines())
 		else:
 			raise ConfigPathError, self.file
@@ -491,15 +468,17 @@ class ConfigFile(ListDict):
 		self.file.writelines(self.dump())
 		self.set_modified(False)
 
+	def write_async(self):
+		operation = self.file.writelines_async(self.dump())
+		# TODO do we need async error handling here ?
+		self.set_modified(False)
+		return operation
+
 
 class ConfigDictFile(ConfigFile, ConfigDict):
 	pass
 
 
-#~ class ConfigListFile(ConfigFile, ConfigList):
-	#~ pass
-
-
 class TextConfigFile(list):
 	'''Like ConfigFile, but just represents a list of lines'''
 
