@@ -39,7 +39,7 @@ from zim.notebook import Path, Page, PageNameError, \
 from zim.stores import encode_filename
 from zim.index import LINK_DIR_BACKWARD
 from zim.config import data_file, config_file, data_dirs, ListDict
-from zim.parsing import url_encode, is_win32_share_re
+from zim.parsing import url_encode, URL_ENCODE_DATA, is_win32_share_re
 from zim.history import History, HistoryRecord
 from zim.gui.pathbar import NamespacePathBar, RecentPathBar, HistoryPathBar
 from zim.gui.pageindex import PageIndex
@@ -47,7 +47,7 @@ from zim.gui.pageview import PageView
 from zim.gui.widgets import Button, MenuButton, \
 	Dialog, ErrorDialog, QuestionDialog, FileDialog, ProgressBarDialog
 from zim.gui.clipboard import Clipboard
-from zim.gui.applications import get_application
+from zim.gui.applications import get_application, CustomToolManager
 
 logger = logging.getLogger('zim.gui')
 
@@ -68,7 +68,7 @@ ui_actions = (
 	('new_page',  'gtk-new', _('_New Page...'), '<ctrl>N', '', False), # T: Menu item
 	('new_sub_page',  'gtk-new', _('New S_ub Page...'), '', '', False), # T: Menu item
 	('open_notebook', 'gtk-open', _('_Open Another Notebook...'), '<ctrl>O', '', True), # T: Menu item
-	('open_new_window', None, _('_Open in New Window'), '', '', True), # T: Menu item
+	('open_new_window', None, _('Open in New _Window'), '', '', True), # T: Menu item
 	('import_page', None, _('_Import Page...'), '', '', False), # T: Menu item
 	('save_page', 'gtk-save', _('_Save'), '<ctrl>S', '', False), # T: Menu item
 	('save_copy', None, _('Save A _Copy...'), '', '', True), # T: Menu item
@@ -93,6 +93,7 @@ ui_actions = (
 	('edit_page_source', 'gtk-edit', _('Edit _Source'), '', '', False), # T: Menu item
 	('show_server_gui', None, _('Start _Web Server'), '', '', True), # T: Menu item
 	('reload_index', None, _('Re-build Index'), '', '', False), # T: Menu item
+	('manage_custom_tools', 'gtk-preferences', _('Custom _Tools'), '', '', True), # T: Menu item
 	('open_page_back', 'gtk-go-back', _('_Back'), '<alt>Left', _('Go page back'), True), # T: Menu item
 	('open_page_forward', 'gtk-go-forward', _('_Forward'), '<alt>Right', _('Go page forward'), True), # T: Menu item
 	('open_page_parent', 'gtk-go-up', _('_Parent'), '<alt>Up', _('Go to parent page'), True), # T: Menu item
@@ -185,6 +186,14 @@ load_zim_stock_icons()
 KEYVAL_ESC = gtk.gdk.keyval_from_name('Escape')
 
 
+def schedule_on_idle(function, args=()):
+	'''Helper function to schedule stuff that can be done later'''
+	def callback():
+		function(*args)
+		return False # delete signal
+	gobject.idle_add(callback)
+
+
 class NoSuchFileError(Error):
 
 	description = _('The file or folder you specified does not exist.\nPlease check if you the path is correct.')
@@ -195,6 +204,26 @@ class NoSuchFileError(Error):
 			# T: Error message, %s will be the file path
 
 
+class RLock(object):
+	'''Kind of re-entrant lock that keeps a stack count'''
+
+	__slots__ = ('count',)
+
+	def __init__(self):
+		self.count = 0
+
+	def __nonzero__(self):
+		return self.count > 0
+
+	def increment(self):
+		self.count += 1
+
+	def decrement(self):
+		if self.count == 0:
+			raise AssertionError, 'BUG: RLock count can not go below zero'
+		self.count -= 1
+
+
 class GtkInterface(NotebookInterface):
 	'''Main class for the zim Gtk interface. This object wraps a single
 	notebook and provides actions to manipulate and access this notebook.
@@ -202,8 +231,6 @@ class GtkInterface(NotebookInterface):
 	Signals:
 	* open-page (page, path)
 	  Called when opening another page, see open_page() for details
-	* save-page (page)
-	  Called when a page is saved
 	* close-page (page)
 	  Called when closing a page, typically just before a new page is opened
 	  and before closing the application
@@ -221,7 +248,6 @@ class GtkInterface(NotebookInterface):
 	# define signals we want to use - (closure type, return type and arg types)
 	__gsignals__ = {
 		'open-page': (gobject.SIGNAL_RUN_LAST, None, (object, object)),
-		'save-page': (gobject.SIGNAL_RUN_LAST, None, (object,)),
 		'close-page': (gobject.SIGNAL_RUN_LAST, None, (object,)),
 		'preferences-changed': (gobject.SIGNAL_RUN_LAST, None, ()),
 		'readonly-changed': (gobject.SIGNAL_RUN_LAST, None, ()),
@@ -237,7 +263,10 @@ class GtkInterface(NotebookInterface):
 		self.preferences_register = ListDict()
 		self.page = None
 		self.history = None
-		self._save_page_in_progress = False
+		self._autosave_lock = RLock()
+			# used to prevent autosave triggering while we are
+			# doing a (async) save, or when we have an error during
+			# saving.
 		self.readonly = False
 		self.usedaemon = usedaemon
 		self.hideonclose = False
@@ -299,6 +328,10 @@ class GtkInterface(NotebookInterface):
 
 		self.load_plugins()
 
+		self._custom_tool_ui_id = None
+		self._custom_tool_actiongroup = None
+		self.load_custom_tools()
+
 		self.uimanager.ensure_update()
 			# prevent flashing when the toolbar is after showing the window
 			# and do this before connecting signal below for accelmap
@@ -314,6 +347,7 @@ class GtkInterface(NotebookInterface):
 
 		gtk.accel_map_get().connect('changed', on_accel_map_changed)
 
+		self.do_preferences_changed()
 
 		# Deal with commandline arguments for notebook and page
 		if notebook:
@@ -360,14 +394,18 @@ class GtkInterface(NotebookInterface):
 			else:
 				self.open_page_home()
 
+		# We schedule the autosave on idle to try to make it impact
+		# the performance of the applciation less. Of course using the
+		# async interface also helps, but we need to account for cases
+		# where asynchronous actions are not supported.
+
 		def autosave():
 			page = self.mainwindow.pageview.get_page()
-			if page.modified:
-				self.save_page(page)
-			return False # remove signal
+			if page.modified and not self._autosave_lock:
+					self.save_page_async(page)
 
 		def schedule_autosave():
-			gobject.idle_add(autosave)
+			schedule_on_idle(autosave)
 			return True # keep ticking
 
 		# older gobject version doesn't know about seconds
@@ -380,6 +418,7 @@ class GtkInterface(NotebookInterface):
 		gtk.main()
 
 	def present(self, page=None, fullscreen=None, geometry=None):
+		'''Present a specific page the main window and/or set window mode'''
 		self.mainwindow.present()
 		if page:
 			if isinstance(page, basestring):
@@ -391,7 +430,17 @@ class GtkInterface(NotebookInterface):
 		elif fullscreen:
 			self.mainwindow.toggle_fullscreen(show=True)
 
+	def toggle_present(self):
+		'''Present main window if it is not on top, but hide if it is.
+		Used by the TrayIcon to toggle visibility of the window.
+		'''
+		if self.mainwindow.is_active():
+			self.mainwindow.hide()
+		else:
+			self.mainwindow.present()
+
 	def hide(self):
+		'''Hide the main window (this is not the same as minimize)'''
 		self.mainwindow.hide()
 
 	def close(self):
@@ -410,8 +459,6 @@ class GtkInterface(NotebookInterface):
 
 		if self.uistate.modified:
 			self.uistate.write()
-			# This is normally done on idle after close_page(), but here no
-			# idle event will follow because we go directly to main_quit()
 
 		self.mainwindow.destroy()
 		gtk.main_quit()
@@ -596,7 +643,7 @@ class GtkInterface(NotebookInterface):
 		'''Signal handler for open-notebook.'''
 
 		def move_away(o, path):
-			if self.page >= path:
+			if path == self.page or self.page.ischild(path):
 				self.open_page_back() \
 				or self.open_page_parent \
 				or self.open_page_home
@@ -604,14 +651,16 @@ class GtkInterface(NotebookInterface):
 		def follow(o, path, newpath, update_links):
 			if self.page == path:
 				self.open_page(newpath)
-			elif self.page > path:
+			elif self.page.ischild(path):
 				newpath = newpath + self.page.relname(path)
 				newpath = Path(newpath.name) # IndexPath -> Path
 				self.open_page(newpath)
 
 		def autosave(o, p):
+			# Here we explicitly do not save async
+			# and also explicitly no need for _autosave_lock
 			page = self.mainwindow.pageview.get_page()
-			if page.modified:
+			if p == page and page.modified:
 				self.save_page(page)
 
 		NotebookInterface.do_open_notebook(self, notebook)
@@ -620,8 +669,8 @@ class GtkInterface(NotebookInterface):
 		notebook.connect('properties-changed', self.on_notebook_properties_changed)
 		notebook.connect('delete-page', autosave) # before action
 		notebook.connect('move-page', autosave) # before action
-		notebook.connect_after('delete-page', move_away)
-		notebook.connect_after('move-page', follow)
+		notebook.connect('deleted-page', move_away)
+		notebook.connect('moved-page', follow)
 
 		# Start a lightweight background check of the index
 		self.notebook.index.update(background=True, checkcontents=False)
@@ -684,7 +733,9 @@ class GtkInterface(NotebookInterface):
 			# redundant calls.
 			return
 		elif self.page:
-			assert self.close_page(self.page)
+			if not self.close_page(self.page):
+				raise AssertionError, 'Could not close page'
+				# assert statement could be optimized away
 
 		logger.info('Open page: %s (%s)', page, path)
 		self.emit('open-page', page, path)
@@ -727,19 +778,15 @@ class GtkInterface(NotebookInterface):
 
 	def do_close_page(self, page):
 		if page.modified:
-			self.save_page(page)
+			self.save_page(page) # No async here -- for now
 
 		current = self.history.get_current()
 		if current == page:
 			current.cursor = self.mainwindow.pageview.get_cursor_pos()
 			current.scroll = self.mainwindow.pageview.get_scroll_pos()
 
-		def save_uistate():
-			if self.uistate.modified:
-				self.uistate.write()
-			return False # only run once
-
-		save_uistate()
+		if self.uistate.modified:
+			schedule_on_idle(self.uistate.write_async)
 
 	def open_page_back(self):
 		record = self.history.get_previous()
@@ -816,38 +863,64 @@ class GtkInterface(NotebookInterface):
 		PageWindow(self, page).show_all()
 
 	def save_page(self, page=None):
-		'''Save 'page', or current page when 'page' is None, by emitting the
-		'save-page' signal. Returns boolean for success.
+		'''Save 'page', or current page when 'page' is None.
+		Returns boolean for success.
 		'''
-		if self._save_page_in_progress:
-			# We need this check as the SavePageErrorDialog has a timer
-			# and auto-save may trigger while we are waiting for that one...
-			return False
+		page = self._save_page_check_page(page)
+		if page is None:
+			return
 
-		self._save_page_in_progress = True
-		try:
-			assert not self.readonly, 'BUG: can not save page when read-only'
-
-			if page is None:
-				page = self.mainwindow.pageview.get_page()
-			assert not page.readonly, 'BUG: can not save read-only page'
-		except Exception, error:
-			SavePageErrorDialog(self, error, page).run()
-			self._save_page_in_progress = False
-			return False
-
-
-		self.emit('save-page', page)
-		self._save_page_in_progress = False
-		return not page.modified
-
-	def do_save_page(self, page):
 		logger.debug('Saving page: %s', page)
 		try:
 			self.notebook.store_page(page)
 		except Exception, error:
 			logger.exception('Failed to save page: %s', page.name)
+			self._autosave_lock.increment()
+				# We need this flag to prevent autosave trigger while we
+				# are showing the SavePageErrorDialog
 			SavePageErrorDialog(self, error, page).run()
+			self._autosave_lock.decrement()
+
+		return not page.modified
+
+	def save_page_async(self, page=None):
+		page = self._save_page_check_page(page)
+		if page is None:
+			return
+
+		logger.debug('Saving page (async): %s', page)
+
+		def callback(ok, error, exc_info, name):
+			# This callback is called back here in the main thread.
+			# We fetch the page again just to be sure in case of strange
+			# edge cases. The SavePageErrorDialog will just take the
+			# current state of the page, not the state that it had in
+			# the async thread. This is done on purpose, current state
+			# is what the user is concerned with anyway.
+			#~ print '!!', ok, exc_info, name
+			if exc_info:
+				page = self.notebook.get_page(Path(name))
+				logger.error('Failed to save page: %s', page.name, exc_info=exc_info)
+				SavePageErrorDialog(self, error, page).run()
+			self._autosave_lock.decrement()
+
+		self._autosave_lock.increment()
+			# Prevent any new auto save to be scheduled while we are
+			# still busy with this call.
+		self.notebook.store_page_async(page, callback=callback, data=page.name)
+
+	def _save_page_check_page(self, page):
+		# Code shared between save_page() and save_page_async()
+		try:
+			assert not self.readonly, 'BUG: can not save page when read-only'
+			if page is None:
+				page = self.mainwindow.pageview.get_page()
+			assert not page.readonly, 'BUG: can not save read-only page'
+		except Exception, error:
+			SavePageErrorDialog(self, error, page).run()
+			return None
+		else:
+			return page
 
 	def save_copy(self):
 		'''Offer to save a copy of a page in the source format, so it can be
@@ -855,20 +928,16 @@ class GtkInterface(NotebookInterface):
 		'''
 		SaveCopyDialog(self).run()
 
-	def save_version(self):
-		pass
-
-	def show_versions(self):
-		from zim.gui.versionsdialog import VersionDialog
-		VersionDialog(self).run()
-
 	def show_export(self):
 		from zim.gui.exportdialog import ExportDialog
 		ExportDialog(self).run()
 
 	def email_page(self):
 		text = ''.join(self.page.dump(format='plain'))
-		url = url_encode('mailto:?subject=%s&body=%s' % (self.page.name, text))
+		url = 'mailto:?subject=%s&body=%s' % (
+			url_encode(self.page.name, mode=URL_ENCODE_DATA),
+			url_encode(text, mode=URL_ENCODE_DATA),
+		)
 		self.open_url(url)
 
 	def import_page(self):
@@ -912,7 +981,18 @@ class GtkInterface(NotebookInterface):
 		RenamePageDialog(self, path=path).run()
 
 	def delete_page(self, path=None):
-		DeletePageDialog(self, path=path).run()
+		if path is None:
+			path = self.get_path_context()
+
+		page = self.notebook.get_page(path)
+		if page.hascontent or page.haschildren:
+			DeletePageDialog(self, path=page).run()
+		else:
+			ErrorDialog(self, (
+				_('Page does not exist'), # T: short error description
+				_('This page does not exist, so it can not be deleted')
+					# T: long error description
+			) ).run()
 
 	def show_properties(self):
 		from zim.gui.propertiesdialog import PropertiesDialog
@@ -936,7 +1016,7 @@ class GtkInterface(NotebookInterface):
 
 	def save_preferences(self):
 		if self.preferences.modified:
-			self.preferences.write()
+			self.preferences.write_async()
 			self.emit('preferences-changed')
 
 	def do_preferences_changed(self):
@@ -944,8 +1024,10 @@ class GtkInterface(NotebookInterface):
 			self.preferences['GtkInterface']['tearoff_menus'] )
 
 	def reload_page(self):
-		if self.page.modified:
-			assert self.save_page(self.page)
+		if self.page.modified \
+		and not self.save_page(self.page):
+			raise AssertionError, 'Could not save page'
+			# assert statement could be optimized away
 		self.notebook.flush_page_cache(self.page)
 		self.open_page(self.notebook.get_page(self.page))
 
@@ -1061,6 +1143,8 @@ class GtkInterface(NotebookInterface):
 		self.spawn('--server', '--gui', self.notebook.uri)
 
 	def reload_index(self):
+		self.mainwindow.pageindex.reload_model()
+			# Flush any sync error in treemodel
 		dialog = ProgressBarDialog(self, _('Updating index'))
 			# T: Title of progressbar dialog
 		dialog.show_all()
@@ -1068,6 +1152,86 @@ class GtkInterface(NotebookInterface):
 		index.update(callback=lambda p: dialog.pulse(p.name))
 		dialog.destroy()
 
+	def manage_custom_tools(self):
+		from zim.gui.customtools import CustomToolManagerDialog
+		CustomToolManagerDialog(self).run()
+		self.load_custom_tools()
+
+	def load_custom_tools(self):
+		manager = CustomToolManager()
+
+		# Remove old actions
+		if self._custom_tool_ui_id:
+			self.uimanager.remove_ui(self._custom_tool_ui_id)
+
+		if self._custom_tool_actiongroup:
+			self.uimanager.remove_action_group(self._custom_tool_actiongroup)
+
+		# Load new actions
+		actions = []
+		for tool in manager:
+			# FIXME probably need to generate stock icon first if icon is file path
+			action = (tool.key, tool.icon, tool.name, '', tool.comment, self.exec_custom_tool)
+			actions.append(action)
+		self._custom_tool_actiongroup = gtk.ActionGroup('custom_tools')
+		self._custom_tool_actiongroup.add_actions(actions)
+
+		menulines = ["<menuitem action='%s'/>\n" % tool.key for tool in manager]
+		toollines = ["<toolitem action='%s'/>\n" % tool.key for tool in manager if tool.showintoolbar]
+		textlines = ["<menuitem action='%s'/>\n" % tool.key for tool in manager if tool.showincontextmenu == 'Text']
+		pagelines = ["<menuitem action='%s'/>\n" % tool.key for tool in manager if tool.showincontextmenu == 'Page']
+		ui = """\
+<ui>
+	<menubar name='menubar'>
+		<menu action='tools_menu'>
+			<placeholder name='custom_tools'>
+			 %s
+			</placeholder>
+		</menu>
+	</menubar>
+	<toolbar name='toolbar'>
+		<placeholder name='tools'>
+		%s
+		</placeholder>
+	</toolbar>
+	<popup name='text_popup'>
+		<placeholder name='tools'>
+		%s
+		</placeholder>
+	</popup>
+	<popup name='page_popup'>
+		<placeholder name='tools'>
+		%s
+		</placeholder>
+	</popup>
+</ui>
+""" % (
+	''.join(menulines), ''.join(toollines),
+	''.join(textlines), ''.join(pagelines)
+)
+
+		self.uimanager.insert_action_group(self._custom_tool_actiongroup, 0)
+		self._custom_tool_ui_id = self.uimanager.add_ui_from_string(ui)
+
+		# TODO also support toolbar, need to add icons to icon factory etc.
+
+	def exec_custom_tool(self, action):
+		manager = CustomToolManager()
+		tool = manager.get_tool(action.get_name())
+		logger.info('Execute custom tool %s', tool.name)
+		args = (self.notebook, self.page, self.mainwindow.pageview)
+		try:
+			if tool.isreadonly:
+				tool.spawn(args)
+			else:
+				tool.run(args)
+				self.reload_page()
+				#~ self.notebook.index.update(background=True)
+				# TODO instead of using run, use spawn and show dialog
+				# with cancel button. Dialog blocks ui.
+		except Exception, error:
+			ErrorDialog(self, error).run()
+
 	def show_help(self, page=None):
 		if page:
 			self.spawn('--manual', page)
@@ -1132,6 +1296,7 @@ class MainWindow(gtk.Window):
 		# hide or destroy the window.
 		def do_delete_event(*a):
 			logger.debug('Action: close (delete-event)')
+			self.hide() # look more responsive
 			ui.close()
 			return True # Do not destroy - let close() handle it
 		self.connect('delete-event', do_delete_event)
@@ -1171,7 +1336,7 @@ class MainWindow(gtk.Window):
 		vbox2.pack_start(self.pathbar_box, False)
 
 		self.pageview = PageView(ui)
-		self.pageview.view.connect(
+		self.pageview.view.connect_after(
 			'toggle-overwrite', self.do_textview_toggle_overwrite)
 		vbox2.add(self.pageview)
 
@@ -1184,24 +1349,6 @@ class MainWindow(gtk.Window):
 		self.statusbar.push(0, '<page>')
 		hbox.add(self.statusbar)
 
-		def update_statusbar(*a):
-			page = self.pageview.get_page()
-			if not page:
-				return
-			label = page.name
-			# TODO if page is read-only
-			if page.modified:
-				label += '*'
-			if self.ui.readonly or page.readonly:
-				label += ' ['+_('readonly')+']' # T: page status in statusbar
-			self.statusbar.pop(0)
-			self.statusbar.push(0, label)
-
-		self.pageview.connect('modified-changed', update_statusbar)
-		self.ui.connect_after('open-page', update_statusbar)
-		self.ui.connect_after('save-page', update_statusbar)
-		self.ui.connect_after('readonly-changed', update_statusbar)
-
 		def statusbar_element(string, size):
 			frame = gtk.Frame()
 			frame.set_shadow_type(gtk.SHADOW_IN)
@@ -1243,6 +1390,18 @@ class MainWindow(gtk.Window):
 		elif fullscreen:
 			self._set_fullscreen = True
 
+	def do_update_statusbar(self, *a):
+		page = self.pageview.get_page()
+		if not page:
+			return
+		label = page.name
+		if page.modified:
+			label += '*'
+		if self.ui.readonly or page.readonly:
+			label += ' ['+_('readonly')+']' # T: page status in statusbar
+		self.statusbar.pop(0)
+		self.statusbar.push(0, label)
+
 	def do_window_state_event(self, event):
 		#~ print 'window-state changed:', event.changed_mask
 		#~ print 'window-state new state:', event.new_window_state
@@ -1569,6 +1728,12 @@ class MainWindow(gtk.Window):
 		self.on_notebook_properties_changed(notebook)
 		notebook.connect('properties-changed', self.on_notebook_properties_changed)
 
+		# Hook up the statusbar
+		self.ui.connect_after('open-page', self.do_update_statusbar)
+		self.ui.connect_after('readonly-changed', self.do_update_statusbar)
+		self.pageview.connect('modified-changed', self.do_update_statusbar)
+		notebook.connect_after('stored-page', self.do_update_statusbar)
+
 	def _set_widgets_visable(self):
 		# Convenience method to switch visibility of all widgets
 		if self._fullscreen:
@@ -1807,10 +1972,10 @@ class NewPageDialog(Dialog):
 		else: title = _('New Page') # T: Dialog title
 
 		Dialog.__init__(self, ui, title,
-			text=_(
+			help_text=_(
 				'Please note that linking to a non-existing page\n'
 				'also creates a new page automatically.'),
-				# T: Dialog text in 'new page' dialog
+			# T: Dialog text in 'new page' dialog
 			path_context=path,
 			fields=[('name', 'page', _('Page Name'), None)], # T: Input label
 			help=':Help:Pages'
@@ -1828,8 +1993,13 @@ class NewPageDialog(Dialog):
 			if page.hascontent or page.haschildren:
 				ErrorDialog(self, _('Page exists')+': %s' % page.name).run() # T: error message
 				return False
-			self.ui.open_page(page)
+
+			template = self.ui.notebook.get_template(page)
+			tree = template.process_to_parsetree(self.ui.notebook, page)
+			page.set_parsetree(tree)
 			self.ui.save_page()
+
+			self.ui.open_page(page)
 			return True
 		else:
 			return False
@@ -1897,8 +2067,11 @@ class MovePageDialog(Dialog):
 			self.path = path
 		assert self.path, 'Need a page here'
 
-		if isinstance(self.path, Page) and self.path.modified:
-			assert self.ui.save_page(self.path)
+		if isinstance(self.path, Page) \
+		and self.path.modified \
+		and not self.ui.save_page(self.path):
+			raise AssertionError, 'Could not save page'
+			# assert statement could be optimized away
 
 		i = self.ui.notebook.index.n_list_links_to_tree(
 					self.path, zim.index.LINK_DIR_BACKWARD )
@@ -2027,22 +2200,31 @@ class AttachFileDialog(FileDialog):
 				# T: Error dialog - %s is the full page name
 			raise Exception, 'Page "%s" does not have a folder for attachments' % self.path
 
+		self.uistate.setdefault('insert_attached_images', True)
+		checkbox = gtk.CheckButton(_('Insert images as link'))
+			# T: checkbox in the "Attach File" dialog
+		checkbox.set_active(self.uistate['insert_attached_images'])
+		self.filechooser.set_extra_widget(checkbox)
+
 	def do_response_ok(self):
 		file = self.get_file()
 		if file is None:
 			return False
+
+		checkbox = self.filechooser.get_extra_widget()
+		self.uistate['insert_attached_images'] = checkbox.get_active()
+			# Similar code in zim.gui.InsertImageDialog
+
+		file.copyto(self.dir)
+		file = self.dir.file(file.basename)
+		pageview = self.ui.mainwindow.pageview
+		if self.uistate['insert_attached_images'] and file.isimage():
+			try:
+				pageview.insert_image(file, interactive=False)
+			except:
+				logger.exception('Could not insert image')
+				pageview.insert_links([file]) # image type not supported?
 		else:
-			file.copyto(self.dir)
-			file = self.dir.file(file.basename)
-			mimetype = file.get_mimetype()
-			pageview = self.ui.mainwindow.pageview
-			if mimetype.startswith('image/'):
-				try:
-					pageview.insert_image(file, interactive=False)
-				except:
-					logger.exception('Could not insert image')
-					pageview.insert_links([file]) # image type not supported?
-			else:
-				pageview.insert_links([file])
-			return True
+			pageview.insert_links([file])
+		return True
 
