@@ -18,6 +18,7 @@ from zim.fs import *
 import zim.errors
 import zim.config
 from zim.notebook import Notebook, Path, PageNameError
+from zim.parsing import link_type
 
 
 logger = logging.getLogger('zim.gui')
@@ -59,6 +60,34 @@ def scrolled_text_view(text=None, monospace=False):
 	return window, textview
 
 
+def gtk_get_style():
+	'''Returns a gtk.Style object for the current theme style.
+	This function is a bit of a hack, but works.
+	'''
+	w = gtk.Window()
+	w.realize()
+	return w.get_style()
+
+
+def rotate_pixbuf(pixbuf):
+	'''If the pixbuf has asociated data for the image rotation
+	(e.g. EXIF for photos) it will rotate the pixbuf to the correct
+	orientation. Returns a new version of the pixbuf or the pixbuf itself.
+	'''
+	# Values for orientation seen in some random snippet in gtkpod
+	o = pixbuf.get_option('orientation')
+	if o: o = int(o)
+	if o == 3: # 180 degrees
+		return pixbuf.rotate_simple(gtk.gdk.PIXBUF_ROTATE_UPSIDEDOWN)
+	elif o == 6: # 270 degrees
+		return pixbuf.rotate_simple(gtk.gdk.PIXBUF_ROTATE_CLOCKWISE)
+	elif o == 9: # 90 degrees
+		return pixbuf.rotate_simple(gtk.gdk.PIXBUF_ROTATE_COUNTERCLOCKWISE)
+	else:
+		# No rotation info, older gtk version, or advanced transpose
+		return pixbuf
+
+
 class Button(gtk.Button):
 	'''This class overloads the constructor of the default gtk.Button
 	class. The purpose is to change the behavior in such a way that stock
@@ -88,6 +117,34 @@ class IconButton(gtk.Button):
 			self.set_relief(gtk.RELIEF_NONE)
 
 
+class IconChooserButton(gtk.Button):
+	'''Button with a stock icon, but no label.'''
+
+	def __init__(self, stock=gtk.STOCK_MISSING_IMAGE):
+		gtk.Button.__init__(self)
+		self.file = None
+		image = gtk.image_new_from_stock(stock, gtk.ICON_SIZE_DIALOG)
+		self.add(image)
+		self.set_alignment(0.5, 0.5)
+
+	def do_clicked(self):
+		dialog = SelectFileDialog(self)
+		dialog.add_filter_images()
+		file = dialog.run()
+		if file:
+			image = self.get_child()
+			size = max(image.size_request()) # HACK to get icon size
+			pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(file.path, size, size)
+			image.set_from_pixbuf(pixbuf)
+			self.file = file
+
+	def get_file(self):
+		return self.file
+
+# Need to register classes defining / overriding gobject signals
+gobject.type_register(IconChooserButton)
+
+
 class SingleClickTreeView(gtk.TreeView):
 	'''Treeview subclass for trees that want single-click behavior,
 	but do allow multiple items to be selected.
@@ -126,7 +183,6 @@ class SingleClickTreeView(gtk.TreeView):
 		def is_rubber_banding_active(self):
 			return False
 
-
 # Need to register classes defining / overriding gobject signals
 gobject.type_register(SingleClickTreeView)
 
@@ -273,20 +329,39 @@ widget "*.zim-statusbar-menubutton" style "zim-statusbar-menubutton-style"
 gobject.type_register(MenuButton)
 
 
-class PageEntry(gtk.Entry):
+class InputEntry(gtk.Entry):
+	'''Sub-class of gtk.Entry with support for highlighting errors'''
 
-	allow_select_root = False
-
-	style = gtk.Entry().get_style() # HACK - how to get default style ?
+	style = gtk_get_style()
 	NORMAL_COLOR = style.base[gtk.STATE_NORMAL]
 	ERROR_COLOR = gtk.gdk.color_parse('#EF7F7F') # light red (derived from Tango style guide)
 
+	def __init__(self):
+		gtk.Entry.__init__(self)
+		self.input_valid = True
+
+	def get_input_valid(self):
+		return self.input_valid
+
+	def set_input_valid(self, valid):
+		'''Set input valid or invalid state'''
+		self.input_valid = valid
+		if valid:
+			self.modify_base(gtk.STATE_NORMAL, self.NORMAL_COLOR)
+		else:
+			self.modify_base(gtk.STATE_NORMAL, self.ERROR_COLOR)
+
+
+class PageEntry(InputEntry):
+
+	allow_select_root = False
+
 	def __init__(self, notebook, path=None, path_context=None):
 		'''Contructor. Needs at least a Notebook to resolve paths.
 		If a context is given this is the reference Path for resolving
 		relative links.
 		'''
-		gtk.Entry.__init__(self)
+		InputEntry.__init__(self)
 		assert notebook, 'Page completion needs a notebook'
 		assert path_context is None or isinstance(path_context, Path)
 		self.notebook = notebook
@@ -331,27 +406,41 @@ class PageEntry(gtk.Entry):
 		text = self.get_text().strip()
 
 		if not text:
-			self.modify_base(gtk.STATE_NORMAL, self.NORMAL_COLOR)
+			self.set_input_valid(True)
 			return
 
 		try:
-			text = Notebook.cleanup_pathname(text)
+			if text != ':' and text != '+':
+				# Clean up, but keep the end ":" chars
+				orig = text
+				text = Notebook.cleanup_pathname(text.lstrip('+'))
+				if orig[0] == ':' and text[0] != ':':
+					text = ':' + text
+				elif orig[0] == '+':
+					text = '+' + text
+				if orig[-1] == ':' and text[-1] != ':':
+					text = text + ':'
+			else:
+				pass
 		except PageNameError:
-			self.modify_base(gtk.STATE_NORMAL, self.ERROR_COLOR)
+			self.set_input_valid(False)
 			return
 		else:
-			self.modify_base(gtk.STATE_NORMAL, self.NORMAL_COLOR)
+			self.set_input_valid(True)
 
 		# Figure out some hint about the namespace
+		anchored = False
 		if ':' in text:
 			# can still have context and start with '+'
 			i = text.rfind(':')
 			completing = text[:i+1]
 			prefix = completing
+			anchored = True
 		elif self.path_context:
 			if text.startswith('+'):
 				completing = ':' + self.path_context.name
 				prefix = '+'
+				anchored = True
 			else:
 				completing = ':' + self.path_context.namespace
 				prefix = ''
@@ -362,6 +451,7 @@ class PageEntry(gtk.Entry):
 		if self.force_child and not completing.startswith('+'):
 			# Needed for new_sub_page - always force child page
 			completing = '+' + completing
+			anchored = True
 
 		# Check if we completed already for this namespace
 		if completing == self._completing:
@@ -379,6 +469,7 @@ class PageEntry(gtk.Entry):
 			except PageNameError:
 				return
 
+		# TODO also add parent namespaces in case text did not contain any ':' (anchored == False)
 		#~ print '!! COMPLETING %s context: %s prefix: %s' % (path, self.path_context, prefix)
 		for p in self.notebook.index.list_pages(path):
 			self.completion_model.append((prefix+p.basename,))
@@ -389,6 +480,21 @@ class NamespaceEntry(PageEntry):
 	allow_select_root = True
 
 
+class LinkEntry(PageEntry):
+	'''Sub-class of PageEntry that also accepts file links and urls'''
+
+	def do_changed(self):
+		text = self.get_text()
+		if text:
+			type = link_type(text)
+			if type == 'page':
+				PageEntry.do_changed(self)
+			else:
+				self.set_input_valid(True)
+		else:
+			self.set_input_valid(True)
+
+
 def format_title(title):
 	'''Formats a window title (in fact just adds " - Zim" to the end).'''
 	assert not title.lower().endswith(' zim')
@@ -449,7 +555,7 @@ class Dialog(gtk.Dialog):
 
 	def __init__(self, ui, title,
 			buttons=gtk.BUTTONS_OK_CANCEL, button=None,
-			text=None, fields=None, help=None,
+			help_text=None, fields=None, help=None,
 			defaultwindowsize=(-1, -1), path_context=None
 		):
 		'''Constructor. 'ui' can either be the main application or some
@@ -465,8 +571,8 @@ class Dialog(gtk.Dialog):
 		item to use instead of the default 'Ok' button (either stock or label
 		can be None).
 
-		Options 'text', 'fields' and 'help' will be past on to add_text(),
-		add_fields() and set_help() respectively.
+		Options 'help_text', 'fields' and 'help' will be past on to
+		add_help_text(), add_fields() and set_help() respectively.
 		'''
 		self.ui = ui
 		self.result = None
@@ -514,7 +620,7 @@ class Dialog(gtk.Dialog):
 		# TODO set Ok button as default widget
 		# see gtk.Window.set_default()
 
-		if text: self.add_text(text)
+		if help_text: self.add_help_text(help_text)
 		if fields: self.add_fields(fields)
 		if help: self.set_help(help)
 
@@ -528,18 +634,26 @@ class Dialog(gtk.Dialog):
 		self.action_area.add(button)
 		self.action_area.set_child_secondary(button, True)
 
-	def add_text(self, text):
-		'''Adds a label in italics. Intended for informational text at the
-		top of the dialog.
+	def add_help_text(self, text):
+		'''Adds a label with an info icon in front of it. Intended for
+		iformational text in dialogs.
 		'''
-		label = gtk.Label()
-		label.set_markup('<i>%s</i>' % _encode_xml(text))
-		self.vbox.pack_start(label, False)
+		hbox = gtk.HBox(spacing=12)
+		self.vbox.pack_start(hbox, False)
+
+		image = gtk.image_new_from_stock(gtk.STOCK_INFO, gtk.ICON_SIZE_BUTTON)
+		image.set_alignment(0.5, 0.0)
+		hbox.pack_start(image, False)
+
+		label = gtk.Label(text)
+		label.set_use_markup(True)
+		label.set_alignment(0.0, 0.0)
+		hbox.add(label)
 
 	def add_fields(self, fields, table=None, trigger_response=True):
 		'''Add a number of fields to the dialog, convenience method to
 		construct simple forms. The argument 'fields' should be a list of
-		field definitions; each definition is a tupple of:
+		field definitions; each definition is a tuple of:
 
 			* The field name
 			* The field type
@@ -557,6 +671,7 @@ class Dialog(gtk.Dialog):
 		will call response_ok(). Set to False if more forms will follow in the
 		same dialog.
 		'''
+		# FIXME FIXME FIXME - this code needs to go in a special class for constructing forms
 		if table is None:
 			table = gtk.Table()
 			table.set_border_width(5)
@@ -580,6 +695,7 @@ class Dialog(gtk.Dialog):
 				v, min, max = value
 				button.set_value(v)
 				button.set_range(min, max)
+				button.set_increments(1,5)
 				self.inputs[name] = button
 				table.attach(button, 1,2, i,i+1)
 			elif type == 'list':
@@ -598,12 +714,14 @@ class Dialog(gtk.Dialog):
 					pass
 				self.inputs[name] = combobox
 				table.attach(combobox, 1,2, i,i+1)
-			elif type in ('string', 'password', 'page', 'namespace', 'dir', 'file', 'image'):
+			elif type in ('string', 'password', 'link', 'page', 'namespace', 'dir', 'file', 'image'):
 				label = gtk.Label(label+': ')
 				label.set_alignment(0.0, 0.5)
 				table.attach(label, 0,1, i,i+1, xoptions=gtk.FILL)
-				if type in ('page', 'namespace'):
-					if type == 'page':
+				if type in ('link', 'page', 'namespace'):
+					if type == 'link':
+						entry = LinkEntry(self.ui.notebook, path_context=self.path_context)
+					elif type == 'page':
 						entry = PageEntry(self.ui.notebook, path_context=self.path_context)
 					else:
 						entry = NamespaceEntry(self.ui.notebook, path_context=self.path_context)
@@ -615,6 +733,13 @@ class Dialog(gtk.Dialog):
 							entry.set_path(value)
 					self.inputs[name] = entry
 					table.attach(entry, 1,2, i,i+1)
+
+					if type == 'link':
+						# FIXME use inline icon for newer versions of Gtk
+						# redundant code from below
+						browse = gtk.Button('_Browse')
+						browse.connect('clicked', self._select_file, (type, entry))
+						table.attach(browse, 2,3, i,i+1, xoptions=gtk.FILL)
 				else:
 					entry = gtk.Entry()
 					entry.zim_type = type
@@ -722,7 +847,11 @@ class Dialog(gtk.Dialog):
 		'''
 		if id == gtk.RESPONSE_OK and not self._no_ok_action:
 			logger.debug('Dialog response OK')
-			self.destroyed = self.do_response_ok()
+			try:
+				self.destroyed = self.do_response_ok()
+			except Exception, error:
+				ErrorDialog(self.ui, error).run()
+				self.destroyed = False
 		else:
 			self.destroyed = True
 
@@ -767,6 +896,13 @@ class ErrorDialog(gtk.MessageDialog):
 		if isinstance(error, zim.errors.Error):
 			msg = error.msg
 			description = error.description
+		elif isinstance(error, EnvironmentError): # e.g. OSError or IOError
+			msg = error.strerror
+			if hasattr(error, 'filename') and error.filename:
+				msg += ': ' + error.filename
+			description = None
+		elif isinstance(error, tuple):
+			msg, description = error
 		else:
 			# Other exception or string
 			msg = unicode(error)
@@ -849,7 +985,7 @@ class FileDialog(Dialog):
 
 	def __init__(self, ui, title, action=gtk.FILE_CHOOSER_ACTION_OPEN,
 			buttons=gtk.BUTTONS_OK_CANCEL, button=None,
-			text=None, fields=None, help=None
+			help_text=None, fields=None, help=None
 		):
 		if button is None:
 			if action == gtk.FILE_CHOOSER_ACTION_OPEN:
@@ -858,7 +994,7 @@ class FileDialog(Dialog):
 				button = (None, gtk.STOCK_SAVE)
 			# else Ok will do
 		Dialog.__init__(self, ui, title,
-			buttons=buttons, button=button, text=text, help=help)
+			buttons=buttons, button=button, help_text=help_text, help=help)
 		if self.uistate['windowsize'] == (-1, -1):
 			self.uistate['windowsize'] = (500, 400)
 			self.set_default_size(500, 400)
