@@ -280,9 +280,9 @@ class Index(gobject.GObject):
 			for link in links:
 				self.cleanup(link)
 
-		self.notebook.connect_after('store-page', on_page_updated)
-		self.notebook.connect_after('move-page', on_page_moved)
-		self.notebook.connect_after('delete-page', lambda o, p: self.delete(p))
+		self.notebook.connect('stored-page', on_page_updated)
+		self.notebook.connect('moved-page', on_page_moved)
+		self.notebook.connect('deleted-page', lambda o, p: self.delete(p))
 
 	def _connect(self):
 		self.db = sqlite3.connect(
@@ -767,6 +767,8 @@ class Index(gobject.GObject):
 		parent = path.parent
 		if isinstance(path, IndexPath):
 			path = self.lookup_data(path)
+			if not path._row:
+				return # apparently it disappeared already
 		else:
 			path = self.lookup_path(path)
 			if not path:
