@@ -21,10 +21,11 @@ import gobject
 
 import zim.fs
 from zim.fs import *
-from zim.errors import Error, SignalExceptionContext, SignalRaiseExceptionContext
+from zim.errors import Error
 from zim.config import ConfigDict, ConfigDictFile, TextConfigFile, HierarchicDict, \
 	config_file, data_dir, user_dirs
 from zim.parsing import Re, is_url_re, is_email_re, is_win32_path_re, link_type, url_encode
+from zim.async import AsyncLock
 import zim.stores
 
 
@@ -203,7 +204,7 @@ def resolve_default_notebook():
 		default = list[0]
 
 	if default:
-		if os.path.isfile(default):
+		if zim.fs.isfile(default):
 			return File(default)
 		else:
 			return Dir(default)
@@ -322,15 +323,27 @@ class Notebook(gobject.GObject):
 	and Index objects on the one hand and the gui application on the other
 
 	This class has the following signals:
-		* store-page (page)
+		* store-page (page) - emitted before actually storing the page
+		* stored-page (page) - emitted after storing the page
 		* move-page (oldpath, newpath, update_links)
+		* moved-page (oldpath, newpath, update_links)
 		* delete-page (path)
+		* deleted-page (path)
 		* properties-changed ()
 
-	All signals are defined with the SIGNAL_RUN_LAST type, so any
-	handler connected normally will run before the actual action.
-	Use "connect_after()" to install handlers after storing, moving
-	or deleting a page.
+	Signals for store, move and delete are defined double with one
+	emitted before the action and the other after the action run
+	succesfully. THis is done this way because exceptions thrown from
+	a signal handler are difficult to handle. For store_async() the
+	'page-stored' signal is emitted after scheduling the store, but
+	potentially before it was really executed.
+
+	Notebook objects have a 'lock' attribute with a AsyncLock object.
+	This lock is used when storing pages. In general this lock is not
+	needed when only reading data from the notebook. However it should
+	be used when doing operations that need a fixed state, e.g.
+	exporting the notebook or when executing version control commands
+	on the storage directory.
 	'''
 
 	# TODO add checks for read-only page in much more methods
@@ -338,8 +351,11 @@ class Notebook(gobject.GObject):
 	# define signals we want to use - (closure type, return type and arg types)
 	__gsignals__ = {
 		'store-page': (gobject.SIGNAL_RUN_LAST, None, (object,)),
+		'stored-page': (gobject.SIGNAL_RUN_LAST, None, (object,)),
 		'move-page': (gobject.SIGNAL_RUN_LAST, None, (object, object, bool)),
+		'moved-page': (gobject.SIGNAL_RUN_LAST, None, (object, object, bool)),
 		'delete-page': (gobject.SIGNAL_RUN_LAST, None, (object,)),
+		'deleted-page': (gobject.SIGNAL_RUN_LAST, None, (object,)),
 		'properties-changed': (gobject.SIGNAL_RUN_FIRST, None, ()),
 	}
 
@@ -366,6 +382,7 @@ class Notebook(gobject.GObject):
 		self.name = None
 		self.icon = None
 		self.config = config
+		self.lock = AsyncLock()
 
 		if dir:
 			assert isinstance(dir, Dir)
@@ -593,7 +610,7 @@ class Notebook(gobject.GObject):
 		'''
 		if href == source:
 			return href.basename
-		elif href > source:
+		elif href.ischild(source):
 			return '+' + href.relname(source)
 		else:
 			parent = source.commonparent(href)
@@ -651,12 +668,16 @@ class Notebook(gobject.GObject):
 		# For file system we should reserve (win32 & posix)
 		# "\", "/", ":", "*", "?", '"', "<", ">", "|"
 
+		# Allowing '%' will cause problems with sql wildcards sooner
+		# or later - also for url decoding ambiguity it is better to
+		# keep this one reserved
+
 		orig = name
 		name = name.replace('_', ' ')
 			# Avoid duplicates with and without '_' (e.g. in index)
 			# Note that leading "_" is stripped, due to strip() below
 
-		for char in ("?", "#", "/", "\\", "*", '"', "<", ">", "|"):
+		for char in ("?", "#", "/", "\\", "*", '"', "<", ">", "|", "%"):
 			if char in name:
 				raise PageNameError, orig
 
@@ -748,22 +769,51 @@ class Notebook(gobject.GObject):
 
 	def get_pagelist(self, path):
 		'''Returns a list of page objects.'''
-		store = self.get_store(path)
-		return store.get_pagelist(path)
+		with self.lock:
+			store = self.get_store(path)
+			list = store.get_pagelist(path)
 		# TODO: add sub-stores in this namespace if any
+		return list
 
 	def store_page(self, page):
 		'''Store a page permanently. Commits the parse tree from the page
 		object to the backend store.
 		'''
 		assert page.valid, 'BUG: page object no longer valid'
-		with SignalExceptionContext(self, 'store-page'):
-			self.emit('store-page', page)
+		self.emit('store-page', page)
+		store = self.get_store(page)
+		store.store_page(page)
+		self.emit('stored-page', page)
 
-	def do_store_page(self, page):
-		with SignalRaiseExceptionContext(self, 'store-page'):
-			store = self.get_store(page)
-			store.store_page(page)
+	def store_page_async(self, page, callback=None, data=None):
+		'''Like store_page but asynchronous. Falls back to store_page
+		when the backend does not support asynchronous operation.
+
+		If you add a callback function it will be called after the
+		page was stored (in the main thread). Callback is called like:
+
+			callback(ok, exc_info, data)
+
+			* 'ok' is True is the page was stored OK
+			* 'error' is an Exception object or None
+			* 'exc_info' is a 3 tuple of sys.exc_info() or None
+			* 'data' is the data given to the constructor
+
+		The callback should be used to do proper error handling if you
+		want to use this interface e.g. from the UI.
+		'''
+		# TODO: make consistent with store-page signal
+
+		# Note that we do not assume here that async function is always
+		# performed by zim.async. Different backends could have their
+		# native support for asynchronous actions. So we do not return
+		# an AsyncOperation object to prevent lock in.
+		# This assumption may change in the future.
+		assert page.valid, 'BUG: page object no longer valid'
+		self.emit('store-page', page)
+		store = self.get_store(page)
+		store.store_page_async(page, self.lock, callback, data)
+		self.emit('stored-page', page)
 
 	def revert_page(self, page):
 		'''Reloads the parse tree from the store into the page object.
@@ -791,64 +841,62 @@ class Notebook(gobject.GObject):
 			raise LookupError, 'Page does not exist: %s' % path.name
 		assert not page.modified, 'BUG: moving a page with uncomitted changes'
 
-		with SignalExceptionContext(self, 'move-page'):
-			self.emit('move-page', path, newpath, update_links)
-
-	def do_move_page(self, path, newpath, update_links):
+		self.emit('move-page', path, newpath, update_links)
 		logger.debug('Move %s to %s (%s)', path, newpath, update_links)
 
-		with SignalRaiseExceptionContext(self, 'move-page'):
-			# Collect backlinks
-			if update_links:
-				from zim.index import LINK_DIR_BACKWARD
-				backlinkpages = set(
+		# Collect backlinks
+		if update_links:
+			from zim.index import LINK_DIR_BACKWARD
+			backlinkpages = set(
+				l.source for l in
+					self.index.list_links(path, LINK_DIR_BACKWARD) )
+			for child in self.index.walk(path):
+				backlinkpages.update(set(
 					l.source for l in
-						self.index.list_links(path, LINK_DIR_BACKWARD) )
-				for child in self.index.walk(path):
-					backlinkpages.update(set(
-						l.source for l in
-							self.index.list_links(path, LINK_DIR_BACKWARD) ))
+						self.index.list_links(path, LINK_DIR_BACKWARD) ))
 
-			# Do the actual move
-			store = self.get_store(path)
-			newstore = self.get_store(newpath)
-			if newstore == store:
-				store.move_page(path, newpath)
-			else:
-				assert False, 'TODO: move between stores'
-				# recursive + move attachments as well
+		# Do the actual move
+		store = self.get_store(path)
+		newstore = self.get_store(newpath)
+		if newstore == store:
+			store.move_page(path, newpath)
+		else:
+			assert False, 'TODO: move between stores'
+			# recursive + move attachments as well
 
-			self.flush_page_cache(path)
-			self.flush_page_cache(newpath)
+		self.flush_page_cache(path)
+		self.flush_page_cache(newpath)
 
-			# Update links in moved pages
-			page = self.get_page(newpath)
-			if page.hascontent:
-				self._update_links_from(page, path)
-				store = self.get_store(page)
-				store.store_page(page)
-				# do not use self.store_page because it emits signals
-			for child in self._no_index_walk(newpath):
-				if not child.hascontent:
+		# Update links in moved pages
+		page = self.get_page(newpath)
+		if page.hascontent:
+			self._update_links_from(page, path)
+			store = self.get_store(page)
+			store.store_page(page)
+			# do not use self.store_page because it emits signals
+		for child in self._no_index_walk(newpath):
+			if not child.hascontent:
+				continue
+			oldpath = path + child.relname(newpath)
+			self._update_links_from(child, oldpath)
+			store = self.get_store(child)
+			store.store_page(child)
+			# do not use self.store_page because it emits signals
+
+		# Update links to the moved page tree
+		if update_links:
+			# Need this indexed before we can resolve links to it
+			self.index.delete(path)
+			self.index.update(newpath)
+			#~ print backlinkpages
+			for p in backlinkpages:
+				if p == path or p.ischild(path):
 					continue
-				oldpath = path + child.relname(newpath)
-				self._update_links_from(child, oldpath)
-				store = self.get_store(child)
-				store.store_page(child)
-				# do not use self.store_page because it emits signals
+				page = self.get_page(p)
+				self._update_links_in_page(page, path, newpath)
+				self.store_page(page)
 
-			# Update links to the moved page tree
-			if update_links:
-				# Need this indexed before we can resolve links to it
-				self.index.delete(path)
-				self.index.update(newpath)
-				#~ print backlinkpages
-				for p in backlinkpages:
-					if p == path or p > path:
-						continue
-					page = self.get_page(p)
-					self._update_links_in_page(page, path, newpath)
-					self.store_page(page)
+		self.emit('moved-page', path, newpath, update_links)
 
 	def _no_index_walk(self, path):
 		'''Walking that can be used when the index is not in sync'''
@@ -874,20 +922,24 @@ class Notebook(gobject.GObject):
 			return
 
 		for tag in tree.getiterator('link'):
-			href = tag.attrib['href']
-			type = link_type(href)
-			if type == 'page':
-				hrefpath = self.resolve_path(href, source=page)
-				oldhrefpath = self.resolve_path(href, source=oldpath)
-				#~ print 'LINK', oldhrefpath, '->', hrefpath
-				if hrefpath != oldhrefpath:
-					if hrefpath >= page and oldhrefpath >= oldpath:
-						#~ print '\t.. Ignore'
-						pass
-					else:
-						newhref = self.relative_link(page, oldhrefpath)
-						#~ print '\t->', newhref
-						self._update_link_tag(tag, newhref)
+			try:
+				href = tag.attrib['href']
+				type = link_type(href)
+				if type == 'page':
+					hrefpath = self.resolve_path(href, source=page)
+					oldhrefpath = self.resolve_path(href, source=oldpath)
+					#~ print 'LINK', oldhrefpath, '->', hrefpath
+					if hrefpath != oldhrefpath:
+						if (hrefpath == page or hrefpath.ischild(page)) \
+						and (oldhrefpath == oldpath or oldhrefpath.ischild(oldpath)):
+							#~ print '\t.. Ignore'
+							pass
+						else:
+							newhref = self.relative_link(page, oldhrefpath)
+							#~ print '\t->', newhref
+							self._update_link_tag(tag, newhref)
+			except:
+				logger.exception('Error while updating link "%s"', href)
 
 		page.set_parsetree(tree)
 
@@ -903,32 +955,35 @@ class Notebook(gobject.GObject):
 			return
 
 		for tag in tree.getiterator('link'):
-			href = tag.attrib['href']
-			type = link_type(href)
-			if type == 'page':
-				hrefpath = self.resolve_path(href, source=page)
-				#~ print 'LINK', hrefpath
-				if hrefpath == oldpath:
-					newhrefpath = newpath
-					#~ print '\t==', oldpath, '->', newhrefpath
-				elif hrefpath > oldpath:
-					rel = hrefpath.relname(oldpath)
-					newhrefpath = newpath + rel
-					#~ print '\t>', oldpath, '->', newhrefpath
-				else:
-					continue
+			try:
+				href = tag.attrib['href']
+				type = link_type(href)
+				if type == 'page':
+					hrefpath = self.resolve_path(href, source=page)
+					#~ print 'LINK', hrefpath
+					if hrefpath == oldpath:
+						newhrefpath = newpath
+						#~ print '\t==', oldpath, '->', newhrefpath
+					elif hrefpath.ischild(oldpath):
+						rel = hrefpath.relname(oldpath)
+						newhrefpath = newpath + rel
+						#~ print '\t>', oldpath, '->', newhrefpath
+					else:
+						continue
 
-				newhref = self.relative_link(page, newhrefpath)
-				self._update_link_tag(tag, newhref)
+					newhref = self.relative_link(page, newhrefpath)
+					self._update_link_tag(tag, newhref)
+			except:
+				logger.exception('Error while updating link "%s"', href)
 
 		page.set_parsetree(tree)
 
 	def rename_page(self, path, newbasename,
 						update_heading=True, update_links=True):
-		'''Rename page to a page in the same namespace but with a new basename.
-		If 'update_heading' is True the first heading in the page will be updated to it's
-		new name.  If 'update_links' is True all links from and to the page will be
-		modified as well.
+		'''Rename page to a page in the same namespace but with a new
+		basename. If 'update_heading' is True the first heading in the
+		page will be updated to it's new name.  If 'update_links' is
+		True all links from and to the page will be modified as well.
 		'''
 		logger.debug('Rename %s to "%s" (%s, %s)',
 			path, newbasename, update_heading, update_links)
@@ -942,24 +997,23 @@ class Notebook(gobject.GObject):
 
 		self.move_page(path, newpath, update_links=update_links)
 		if update_heading:
+			import zim.parsing
 			page = self.get_page(newpath)
 			tree = page.get_parsetree()
 			if not tree is None:
-				tree.set_heading(newbasename.title())
+				tree.set_heading(zim.parsing.title(newbasename))
 				page.set_parsetree(tree)
 				self.store_page(page)
 
 		return newpath
 
 	def delete_page(self, path):
-		with SignalExceptionContext(self, 'delete-page'):
-			self.emit('delete-page', path)
-
-	def do_delete_page(self, path):
-		with SignalRaiseExceptionContext(self, 'delete-page'):
-			store = self.get_store(path)
-			store.delete_page(path)
-			self.flush_page_cache(path)
+		self.emit('delete-page', path)
+		store = self.get_store(path)
+		store.delete_page(path)
+		self.flush_page_cache(path)
+		path = Path(path.name)
+		self.emit('deleted-page', path)
 
 	def resolve_file(self, filename, path):
 		'''Resolves a file or directory path relative to a page. Returns a
@@ -1174,6 +1228,9 @@ class Notebook(gobject.GObject):
 gobject.type_register(Notebook)
 
 
+import warnings
+
+
 class Path(object):
 	'''This is the parent class for the Page class. It contains the name
 	of the page and is used instead of the actual page object by methods
@@ -1203,6 +1260,9 @@ class Path(object):
 	def __repr__(self):
 		return '<%s: %s>' % (self.__class__.__name__, self.name)
 
+	def __hash__(self):
+		return self.name.__hash__()
+
 	def __eq__(self, other):
 		'''Paths are equal when their names are the same'''
 		if isinstance(other, Path):
@@ -1213,26 +1273,30 @@ class Path(object):
 	def __ne__(self, other):
 		return not self.__eq__(other)
 
+	def __add__(self, name):
+		'''"path + name" is an alias for path.child(name)'''
+		return self.child(name)
+
 	def __lt__(self, other):
 		'''`self < other` evaluates True when self is a parent of other'''
+		warnings.warn('Usage of Path.__lt__ is deprecated', DeprecationWarning, 2)
 		return self.isroot or other.name.startswith(self.name+':')
 
 	def __le__(self, other):
 		'''`self <= other` is True if `self == other or self < other`'''
+		warnings.warn('Usage of Path.__le__ is deprecated', DeprecationWarning, 2)
 		return self.__eq__(other) or self.__lt__(other)
 
 	def __gt__(self, other):
 		'''`self > other` evaluates True when self is a child of other'''
+		warnings.warn('Usage of Path.__gt__ is deprecated', DeprecationWarning, 2)
 		return other.isroot or self.name.startswith(other.name+':')
 
 	def __ge__(self, other):
 		'''`self >= other` is True if `self == other or self > other`'''
+		warnings.warn('Usage of Path.__ge__ is deprecated', DeprecationWarning, 2)
 		return self.__eq__(other) or self.__gt__(other)
 
-	def __add__(self, name):
-		'''"path + name" is an alias for path.child(name)'''
-		return self.child(name)
-
 	@property
 	def parts(self):
 		return self.name.split(':')
@@ -1299,6 +1363,10 @@ class Path(object):
 		else: # we are the top level root namespace
 			return Path(name)
 
+	def ischild(self, parent):
+		'''Returns True if this path is a child of 'parent' '''
+		return parent.isroot or self.name.startswith(parent.name + ':')
+
 	def commonparent(self, other):
 		parent = []
 		parts = self.parts
@@ -1358,7 +1426,11 @@ class Page(Path):
 		if self._parsetree:
 			return self._parsetree.hascontent
 		elif self._ui_object:
-			return self._ui_object.get_parsetree().hascontent
+			tree = self._ui_object.get_parsetree()
+			if tree:
+				return tree.hascontent
+			else:
+				return False
 		else:
 			try:
 				hascontent = self._source_hascontent()
@@ -1516,7 +1588,9 @@ class IndexPage(Page):
 		add_namespace(self)
 		builder.end('page')
 
-		return zim.formats.ParseTree(builder.close())
+		tree = zim.formats.ParseTree(builder.close())
+		#~ print "!!!", tree.tostring()
+		return tree
 
 
 class Link(object):
