@@ -5,6 +5,7 @@
 '''This module contains utilities for parsing strings and text'''
 
 import re
+import string
 
 
 def split_quoted_strings(string, unescape=True):
@@ -36,38 +37,131 @@ def unescape_quoted_string(string):
 		string = escape_re.sub(replace, string)
 	return string
 
+# URL encoding / decoding is a bit more tricky than it seems:
+#
+# === From man 7 url ===
+# Reserved chars:   ; / ? : @ & = + $ ,
+# Unreserved chars: A-Z a-z 0-9 - _ . ! ~ * ' ( )
+# although heuristics could have a problem with . ! or ' at end of url
+# All other chars are not allowed and need escaping
+# Unicode chars need to be encoded as utf-8 and then as several escapes
+#
+# === Usage ===
+# Encode all - encode all chars
+#   e.g. for encoding parts of a file:// uri
+#   for encoding data for mailto:?subject=...
+#	return ascii
+# Encode path - encode all except /
+#   convenience method for file paths
+#	return ascii
+# Encode readable - encode space & utf-8, keep other escapes
+#	for pageview -> external (e.g. clipboard)
+#	assume reserved is (still) encoded properly
+#	return ascii
+# Decode all - decode all chars
+#   e.g. for decoding file:// uris
+#	return unicode
+# Decode readable - decode space, utf-8, keep other escapes
+#   for source / external (e.g. clipboard) -> pageview
+#	assume it is encoded properly to start with
+#	return unicode
+#
+# space is really just ' ', other whithespace characters like tab or
+# newline should not appear in the first place - so do not facilitate
+# them.
+#
+# In wiki source we use fully escaped URLs. In theory we could allow
+# for utf-8 characters, but this adds complexity. Also it could break
+# external usage of the text files.
+#
+# === From man 7 utf-8 ===
+# * The classic US-ASCII characters are encoded simply as bytes 0x00 to 0x7f
+# * All UCS characters > 0x7f are encoded as a multi-byte sequence
+#   consisting only of bytes in the range 0x80 to 0xfd, so no ASCII byte
+#   can appear as part of another character
+# * The bytes 0xfe and 0xff are never used in the UTF-8 encoding.
+#
+# So checking ranges makes sure utf-8 is really outside of ascii set,
+# and does not e.g. include "%".
+
+URL_ENCODE_DATA = 0 # all
+URL_ENCODE_PATH = 1	# all except '/'
+URL_ENCODE_READABLE = 2 # only space and utf-8
+
+_url_encode_re = re.compile(r'[^A-Za-z0-9\-_\.!~*\'\(\)]') # unreserved
+_url_encode_path_re = re.compile(r'[^A-Za-z0-9\-_\.!~*\'\(\)/]') # unreserved + /
+
+def _url_encode(match):
+	return '%%%02X' % ord(match.group(0))
+
+def _url_encode_readable(match):
+	i = ord(match.group(0))
+	if i == 32 or i > 127: # space or utf-8
+		return '%%%02X' % i
+	else: # do not encode
+		return match.group(0)
+
+def url_encode(url, mode=URL_ENCODE_PATH):
+	'''Replaces non-standard characters in urls with hex codes.
+
+	Mode can be:
+		URL_ENCODE_DATA - encode all un-safe chars
+		URL_ENCODE_PATH - encode all un-safe chars except '/'
+		URL_ENCODE_READABLE - encode whitespace and all unicode characters
+
+	The mode URL_ENCODE_READABLE can be applied to urls that are already
+	encoded because they do not touch the "%" character. The modes
+	URL_ENCODE_DATA and URL_ENCODE_PATH can only be applied to strings
+	that are known not to be encoded.
+
+	The encoded URL is returned as an ASCII string.
+	'''
+	url = url.encode('utf-8') # unicode -> utf-8, so encode one byte at a time
+
+	if mode == URL_ENCODE_DATA:
+		return _url_encode_re.sub(_url_encode, url)
+	elif mode == URL_ENCODE_PATH:
+		return _url_encode_path_re.sub(_url_encode, url)
+	elif mode == URL_ENCODE_READABLE:
+		return _url_encode_re.sub(_url_encode_readable, url)
+	else:
+		assert False, 'BUG: Unknown url encoding mode'
+
 
-_url_encode_re = re.compile(r'[^A-Za-z0-9\-\_\.\!\~\*\'\(\)\/\:]')
-	# url encoding - char set from man uri(7), see relevant rfc
-	# added '/' and ':' to char set for readability of uris
 _url_decode_re = re.compile('%([a-fA-F0-9]{2})')
 
-_unencoded_url_re = re.compile('\s|%(?![a-fA-F0-9]{2})')
-	# Not sure if everybody else out there uses the same char set,
-	# but whitespace in an url is a sure sign it is not encoded.
-	# (Also whitespace is the most problematic for wiki syntax...)
-	# Otherwise seeing a "%" without hex behind it also a good sign.
+def _url_decode(match):
+	return chr(int(match.group(1), 16))
 
-def url_encode(url):
-	'''Replaces non-standard characters in urls with hex codes.
-	This function uses some heuristics to detect when an url is encoded
-	already and avoid double-encoding it.
-	'''
-	url = url.encode('utf-8') # unicode -> utf-8
-	if not ('%' in url and not _unencoded_url_re.search(url)):
-		url = _url_encode_re.sub(lambda m: '%%%X' % ord(m.group(0)), url)
-	return url
+def _url_decode_readable(match):
+	i = int(match.group(1), 16)
+	if i == 32 or i > 127: # space or utf-8
+		return chr(i)
+	else: # do not decode
+		return match.group(0)
 
-
-def url_decode(url):
+def url_decode(url, mode=URL_ENCODE_PATH):
 	'''Replace url-encoding hex sequences with their proper characters.
-	This function uses some heuristics to detect when an url is not
-	encoded in the first place already and avoid double-decoding it.
+
+	Mode can be:
+		URL_ENCODE_DATA - decode all chars
+		URL_ENCODE_PATH - same as URL_ENCODE_DATA
+		URL_ENCODE_READABLE - decode only whitespace and unicode characters
+
+	The mode URL_ENCODE_READABLE will not decode any other characters,
+	so urls decoded with these modes can still contain escape sequences.
+	They are save fo usage within zim, but should be re-encoded with
+	URL_ENCODE_READABLE before handing them to an external program.
+
+	The result is returned as a unicode string.
 	'''
-	if '%' in url and not _unencoded_url_re.search(url):
-		url = _url_decode_re.sub(lambda m: chr(int(m.group(1), 16)), url)
-	url = url.decode('utf-8') # utf-8 -> unicode
-	return url
+	url = url.encode('utf-8') # in case url is already unicode
+	if mode in (URL_ENCODE_DATA, URL_ENCODE_PATH):
+		return _url_decode_re.sub(_url_decode, url).decode('utf-8')
+	elif mode == URL_ENCODE_READABLE:
+		return _url_decode_re.sub(_url_decode_readable, url).decode('utf-8')
+	else:
+		assert False, 'BUG: Unknown url encoding mode'
 
 
 _parse_date_re = re.compile(r'(\d{1,4})\D(\d{1,2})(?:\D(\d{1,4}))?')
@@ -113,6 +207,24 @@ def parse_date(string):
 		return None
 
 
+# These sets adjust to the current locale - so not same as "[a-z]" ..
+# Must be kidding - no classes for this in the regex engine !?
+_classes = {'upper': string.uppercase}
+upper_re = re.compile(r'[%(upper)s]' % _classes)
+del _classes
+
+def title(string):
+	'''Slightly smarter version of str.title(). Does not "downgrade"
+	words that already have upper case in it.
+	'''
+	def titleword(match):
+		word = match.group(0)
+		if upper_re.search(word): return word
+		else: return word.title()
+
+	return re.sub(r'\w+', titleword, string, re.U)
+
+
 class Re(object):
 	'''Wrapper around regex pattern objects which memorizes the
 	last match object and gives list access to it's capturing groups.
@@ -203,10 +315,10 @@ is_url_re = Re('^(\w[\w\+\-\.]+)://')
 is_email_re = Re('^mailto:|^\S+\@\S+\.\w+$')
 	# "mailto:" address
 	# name "@" host
-is_path_re = Re(r'^(/|\.\.?[/\\]|~.*[/\\]|[A-Z]:\\)')
+is_path_re = Re(r'^(/|\.\.?[/\\]|~.*[/\\]|[A-Za-z]:\\)')
 	# / ~/ ./ ../ ~user/  .\ ..\ ~\ ~user\
 	# X:\
-is_win32_path_re = Re(r'^[A-Z]:[\\/]')
+is_win32_path_re = Re(r'^[A-Za-z]:[\\/]')
 	# X:\ (or X:/)
 is_win32_share_re = Re(r'^(\\\\[^\\]+\\.+|smb://)')
 	# \\host\share
@@ -214,6 +326,7 @@ is_win32_share_re = Re(r'^(\\\\[^\\]+\\.+|smb://)')
 is_interwiki_re = Re('^(\w[\w\+\-\.]+)\?(.*)')
 	# identifyer "?" path
 
+
 _classes = {'c': r'[^\s"<>\']'} # limit the character class a bit
 url_re = Re(r'''(
 	\b \w[\w\+\-\.]+:// %(c)s* \[ %(c)s+ \] (?: %(c)s+ [\w/] )?  |
@@ -245,10 +358,10 @@ class TextBuffer(list):
 	internally.
 	'''
 
-	def get_lines(self):
+	def get_lines(self, end_with_newline=True):
 		'''Returns a proper list of lines'''
 		lines = ''.join(self).splitlines(True)
-		if lines and not lines[-1].endswith('\n'):
+		if end_with_newline and lines and not lines[-1].endswith('\n'):
 			lines[-1] += '\n'
 		return lines
 
